import cython
import os
from win.path import split


@cython.exceptval(check=False)
@cython.cfunc
def removedirs(path: str) -> cython.void:
    while path:
        try:
            os.rmdir(path)
        except OSError:
            break

        path = split(path)[0]


@cython.exceptval(check=False)
@cython.ccall
def multi_removedirs(paths: list) -> cython.void:
    [removedirs(path) for path in paths]



from os.path import *
import cython
import os


# @cython.exceptval(check=False)
@cython.ccall
def normpath(path: str) -> str:
    special_prefixes: tuple = ('\\\\.\\', '\\\\?\\')

    if path.startswith(special_prefixes):
        return path

    prefix: str

    path = path.replace('/', '\\')
    prefix, path = splitdrive(path)

    if path.startswith('\\'):
        prefix: str = f'{prefix}\\'
        path: str = path.lstrip('\\')

    comps: list = path.split('\\')
    i: cython.int = 0

    while i < len(comps):
        if not comps[i] or comps[i] == '.':
            del comps[i]

        elif comps[i] == '..':
            if i > 0 and comps[i - 1] != '..':
                del comps[i - 1:i + 1]
                i -= 1
            elif i == 0 and prefix.endswith('\\'):
                del comps[i]
            else:
                i += 1

        else:
            i += 1

    if not prefix and not comps:
        comps.append('.')

    return prefix + '\\'.join(comps)


# @cython.exceptval(check=False)
@cython.ccall
def _abspath_fallback(path: str) -> str:
    if not isabs(path):
        cwd: str = os.getcwd()
        path = join(cwd, path)
    return normpath(path)


# @cython.exceptval(check=False)
@cython.ccall
def normcase(path: str) -> str:
    path = path.replace('/', '\\')
    path = path.lower()
    return path


# @cython.exceptval(check=False)
@cython.ccall
def split(path: str) -> tuple:
    last_slash: cython.int = path.rfind('\\')

    if last_slash == -1:
        if ':' in path:
            return path[:2], path[2:]
        return '', path

    base: str = path[:last_slash]
    tail: str = path[last_slash + 1:]

    if base[-1] == ':':
        base = f'{base}\\'

    return base, tail


# @cython.exceptval(check=False)
@cython.ccall
def splitdrive(path: str) -> tuple:
    colon_loc: cython.int = path.find(':')

    if colon_loc == -1:
        unc_base_ind: cython.int = path.find('\\', 2)

        if not path or path[0] != '\\':
            return '', path

        unc_path_ind: cython.int = path.find('\\', unc_base_ind + 1)
        result = path[:unc_path_ind], path[unc_path_ind:]
        return result

    result = path[:colon_loc + 1], path[colon_loc + 1:]
    return result


# @cython.exceptval(check=False)
@cython.ccall
def isabs(path: str) -> cython.bint:
    if path.startswith('\\\\?\\'):
        return True

    path = splitdrive(path)[1]
    return path and path[0] == '\\'


def join(path: str, *paths) -> str:
    joined: str = '\\'.join(paths)
    path = path.rstrip('\\')

    if path[-1] == ':':
        return f"{path}{joined}"
    return f"{path}\\{joined}"


# @cython.exceptval(check=False)
@cython.ccall
def splitext(path: str) -> tuple:
    ext_ind: cython.int = path.rfind('.')
    ext: str = path[ext_ind:]

    if ext_ind == -1 or '\\' in ext:
        return (path, '')

    return path[:ext_ind], ext


# @cython.exceptval(check=False)
@cython.ccall
def basename(path: str) -> str:
    return split(path)[1]


# @cython.exceptval(check=False)
@cython.ccall
def dirname(path: str) -> str:
    return split(path)[0]


# @cython.exceptval(check=False)
@cython.ccall
def ismount(path: str) -> cython.bint:
    path = abspath(path)
    return len(path) == 2 and path[1] == ':'


# @cython.exceptval(check=False)
@cython.ccall
def expanduser(path: str) -> str:
    if path.startswith('~\\'):
        userprofile: str = os.environ.get('USERPROFILE', '')
        return f'{userprofile}\\{path[2:]}'

    if path.startswith('~'):
        sep_loc: cython.int = path.find('\\')
        username: str = path[1:None if sep_loc == -1 else sep_loc]
        userprofile = os.environ.get('USERPROFILE', '')

        if os.environ.get('USERNAME') != username:
            home: str = split(userprofile)[0]
            return f'{home}\\{path[1:]}'

        return f'{userprofile}\\{path[1:]}'

    return path


# @cython.exceptval(check=False)
@cython.ccall
def relpath(tail: str, root=None) -> str:
    if root is None:
        root: str = os.getcwd()

    if tail.startswith(root):
        return tail[len(root) + 1:]

    i: cython.int
    chars: tuple
    tail_char: str
    root_char: str

    for i, chars in enumerate(zip(tail, root)):
        tail_char, root_char = chars
        if tail_char != root_char:
            break

    dots: str = "..\\" * (root.count('\\') - tail[:i].count('\\') + 1)
    return f'{dots}{tail[i:]}'


# @cython.exceptval(check=False)
@cython.ccall
def commonprefix(paths) -> str:
    min_path: str = min(paths)
    max_path: str = max(paths)

    i: cython.int = 0
    char: str

    for char in min_path:
        if char != max_path[i]:
            return min_path[:i]
        i += 1

    return min_path


@cython.exceptval(check=False)
@cython.ccall
def commonpath(paths) -> str:
    splitted: list = [path.split('\\') for path in paths]

    min_splitted: list = min(splitted)
    max_splitted: list = max(splitted)

    index: cython.int
    path: str
    path2: str
    result: str

    for index, (path, path2) in enumerate(zip(min_splitted, max_splitted)):
        if path != path2:
            result = '\\'.join(min_splitted[:index])

            try:
                if result[-1] == ':':
                    return result + '\\'
            except IndexError:
                return result
            return result

    return '\\'.join(min_splitted)


try:
    from nt import _getfullpathname

    def abspath(path: str) -> str:
        try:
            return _getfullpathname(normpath(path))
        except:
            return _abspath_fallback(path)

except ImportError:
    abspath = _abspath_fallback


# @cython.exceptval(check=False)
@cython.ccall
def multi_split(paths) -> list:
    return [split(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_normpath(paths) -> list:
    return [normpath(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_normcase(paths) -> list:
    return [normcase(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_splitdrive(paths) -> list:
    return [splitdrive(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_isabs(paths) -> list:
    return [isabs(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_join(paths) -> list:
    return [join(*path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_splitext(paths) -> list:
    return [splitext(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_basename(paths) -> list:
    return [basename(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_dirname(paths) -> list:
    return [dirname(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_relpath(paths) -> list:
    return [relpath(*path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_expanduser(paths) -> list:
    return [expanduser(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_ismount(paths) -> list:
    return [ismount(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_abspath(paths) -> list:
    return [abspath(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_commonprefix(paths) -> list:
    return [commonprefix(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_commonpath(paths) -> list:
    return [commonpath(path) for path in paths]



from setuptools import setup, Extension
import platform

try:
    from Cython.Build import cythonize
    USE_CYTHON = True
except ImportError:
    USE_CYTHON = False

file_ext = 'py' if USE_CYTHON else 'c'
platform_folder = 'win' if platform.system() == 'Windows' else 'unix'

ext_modules = [
    Extension("faster_os", [f"faster_os.{file_ext}"]),
    Extension("faster_os.path", [f"{platform_folder}/path.{file_ext}"]),
    Extension("faster_os.generic", [f'{platform_folder}/generic.{file_ext}']),
]

if USE_CYTHON:
    ext_modules = cythonize(
        ext_modules,
        compiler_directives={
            'language_level': "3",
        },
    )

with open("README.md", encoding='utf8') as file:
    long_description = file.read()

setup(
    name="faster-os",
    version="0.0.11",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        "License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)",
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 3.8",
        'Intended Audience :: Developers',
        'Programming Language :: Cython',
        'Topic :: Office/Business',
        'Topic :: Software Development',
        'Topic :: Software Development :: Libraries',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: System',
        'Topic :: System :: Filesystems',
        'Topic :: System :: Operating System',
        'Topic :: Utilities',
    ],
    license="MPL 2.0",
    author_email='contact@abtco.us',
    author="American Best Technologies Company",
    description="Up to 6700% faster OS module.",
    packages=['unix', 'win'],
    package_dir={'faster_os': 'faster_os'},
    long_description=long_description,
    project_urls={
        "Bug Tracker":
        "https://github.com/American-Best-Technologies-Company/faster-os/issues",
        "Our website": 'http://www.abtco.us',
        "Used by MyQuickMac": 'http://www.myquickmac.com',
    },
    long_description_content_type='text/markdown',
    url="https://github.com/American-Best-Technologies-Company/faster-os",
    ext_modules=ext_modules,
    install_requires=[],
)
"""
faster-than
os
optimized
speedup
speededup
faster
path
file
directory
paths manipulations
files processing
data processing
"""



from os.path import *
import cython
import pwd
import os

try:
    from posix import _path_normpath
except ImportError:

    def normpath(path: str) -> str:
        sep: str = '/'
        empty: str = ''
        dot: str = '.'
        dotdot: str = '..'

        if not path:
            return '.'

        initial_slashes: cython.bint = path.startswith('/')

        if (initial_slashes and path.startswith(sep * 2)
                and not path.startswith(sep * 3)):
            initial_slashes = 2

        comps: list = path.split(sep)
        new_comps: list = []

        for comp in comps:
            if comp in (empty, dot):
                continue
            if (comp != dotdot or (not initial_slashes and not new_comps)
                    or (new_comps and new_comps[-1] == dotdot)):
                new_comps.append(comp)
            elif new_comps:
                new_comps.pop()

        comps = new_comps
        path = sep.join(comps)
        if initial_slashes:
            path = sep * initial_slashes + path

        return path or dot

else:

    def normpath(path: str) -> str:
        return _path_normpath(path) or "."


@cython.exceptval(check=False)
@cython.ccall
def abspath(path: str) -> str:
    if not isabs(path):
        cwd: str = os.getcwd()
        path = join(cwd, path)
    return normpath(path)


@cython.exceptval(check=False)
@cython.ccall
def isabs(path: str) -> cython.bint:
    return path.startswith('/')


@cython.exceptval(check=False)
@cython.ccall
def normcase(path: str) -> str:
    return path


@cython.exceptval(check=False)
@cython.ccall
def split(path: str) -> tuple:
    last_slash: cython.int = path.rfind('/')

    if last_slash == -1:
        return '', path

    if last_slash == 0:
        return '/', path[1:]

    base: str = path[:last_slash]
    tail: str = path[last_slash + 1:]

    return base, tail


@cython.exceptval(check=False)
@cython.ccall
def splitdrive(path: str) -> tuple:
    return '', path


def join(path: str, *paths: list) -> str:
    joined: str = '/'.join(paths)
    if not path:
        return joined

    path = path.rstrip('/')
    return f"{path}/{joined}"


@cython.exceptval(check=False)
@cython.ccall
def splitext(path: str) -> tuple:
    ext_ind: cython.int = path.rfind('.')
    ext: str = path[ext_ind:]

    if ext_ind == -1 or '/' in ext:
        return (path, '')

    return (path[:ext_ind], ext)


@cython.exceptval(check=False)
@cython.ccall
def basename(path: str) -> str:
    return split(path)[1]


@cython.exceptval(check=False)
@cython.ccall
def dirname(path: str) -> str:
    return split(path)[0]


@cython.exceptval(check=False)
@cython.ccall
def ismount(path: str) -> cython.bint:
    return path == '/'


@cython.exceptval(check=False)
@cython.ccall
def expanduser(path: str) -> str:
    if path.startswith('~/'):
        home: list = [
            os.environ.get('HOMEPATH'),
            os.environ.get('HOME'),
        ]

        try:
            home: str = list(filter(bool, home))[0]
        except:
            home: str = f'/home/{os.environ.get("USERNAME")}'

        return f'{home}/{path[2:]}'

    if path.startswith('~'):
        sep_loc: cython.int = path.find('/')
        username: str = path[1:None if sep_loc == -1 else sep_loc]

        try:
            user_path: str = pwd.getpwnam(username).pw_dir
            return f'{user_path}{path[len(username) + 1:]}'
        except:
            return path

    return path


@cython.exceptval(check=False)
@cython.ccall
def relpath(tail: str, root=None) -> str:
    if root is None:
        root: str = os.getcwd()

    if not root:
        return f'{"/.." * (tail.count("/") + 1)}{tail}'[1:]

    if tail.startswith(root):
        return tail[len(root) + 1:]

    for i, chars in enumerate(zip(tail, root)):
        tail_char, root_char = chars
        if tail_char != root_char:
            break

    dots: str = "../" * (root.count('/') - tail[:i].count('/') + 1)
    return f'{dots}{tail[i:]}'


@cython.exceptval(check=False)
@cython.ccall
def commonprefix(paths: list) -> str:
    min_path: str = min(paths)
    max_path: str = max(paths)

    for i, char in enumerate(min_path):
        if char != max_path[i]:
            return min_path[:i]

    return min_path


@cython.exceptval(check=False)
@cython.ccall
def commonpath(paths, lower=False) -> str:
    if lower:
        paths: list = [path.lower() for path in paths]

    splitted: list = [path.split('/') for path in paths]

    min_splitted: list = min(splitted)
    max_splitted: list = max(splitted)

    for index, (path, path2) in enumerate(zip(min_splitted, max_splitted)):
        if path != path2:
            return '/' if min_splitted[:index] == [''] else '/'.join(min_splitted[:index])
    return '/'.join(min_splitted)


# @cython.exceptval(check=False)
@cython.ccall
def multi_split(paths) -> list:
    return [split(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_normpath(paths) -> list:
    return [normpath(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_normcase(paths) -> list:
    return [normcase(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_splitdrive(paths) -> list:
    return [splitdrive(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_isabs(paths) -> list:
    return [isabs(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_join(paths) -> list:
    return [join(*path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_splitext(paths) -> list:
    return [splitext(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_basename(paths) -> list:
    return [basename(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_dirname(paths) -> list:
    return [dirname(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_relpath(paths) -> list:
    return [relpath(*path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_expanduser(paths) -> list:
    return [expanduser(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_ismount(paths) -> list:
    return [ismount(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_abspath(paths) -> list:
    return [abspath(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_commonprefix(paths) -> list:
    return [commonprefix(path) for path in paths]


# @cython.exceptval(check=False)
@cython.ccall
def multi_commonpath(paths) -> list:
    return [commonpath(path) for path in paths]
